"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = cli;

var _fs = require("fs");

var _stream = require("stream");

var _yargs = _interopRequireDefault(require("yargs"));

var _debug = _interopRequireDefault(require("debug"));

var _ = _interopRequireDefault(require("."));

var _commands = _interopRequireDefault(require("./commands"));

var _file = _interopRequireDefault(require("./file"));

var _package = require("../package.json");

var _settings = _interopRequireDefault(require("./settings"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function cli(errlog) {
  var args = _yargs["default"].env('EZS').usage('Usage: $0 [options] [<file>|<directory>] [<file2> <file3> ...]').version(_package.version).options({
    verbose: {
      alias: 'v',
      "default": false,
      describe: 'Enable debug mode with DEBUG=ezs',
      type: 'boolean'
    },
    daemon: {
      alias: 'd',
      describe: 'Launch daemon on a directory containing commands script',
      type: 'string'
    },
    slave: {
      alias: 'l',
      describe: 'Launch slave daemon to execute remote commands',
      type: 'boolean'
    },
    server: {
      alias: 's',
      describe: 'Server to dispach commands',
      type: 'string'
    },
    env: {
      alias: 'e',
      "default": false,
      describe: 'Execute commands with environement variables as input',
      type: 'boolean'
    },
    file: {
      alias: 'f',
      "default": false,
      describe: 'Execute commands with a file as input',
      type: 'string'
    }
  }).epilogue('for more information, find our manual at https://github.com/touv/node-ezs');

  var argv = args.argv;

  if (argv.verbose) {
    _debug["default"].enable('ezs');
  }

  if (argv.daemon) {
    var serverPath;

    try {
      serverPath = (0, _fs.realpathSync)(argv.daemon);
    } catch (e) {
      errlog("Error: ".concat(argv.daemon, " doesn't exists."));
      process.exit(1);
    }

    (0, _debug["default"])('ezs')("Serving ".concat(serverPath, " with ").concat(_settings["default"].nShards, " shards"));
    return _["default"].createCluster(_settings["default"].port, serverPath);
  }

  if (argv.slave) {
    (0, _debug["default"])('ezs')("Waiting for remote commands with ".concat(_settings["default"].nShards, " shards"));
    return _["default"].createCluster(_settings["default"].port, false);
  }

  if (argv._.length === 0) {
    _yargs["default"].showHelp();

    return process.exit(1);
  }

  var input;

  if (argv.env) {
    (0, _debug["default"])('ezs')('Reading environment variables...');
    input = new _stream.PassThrough(_["default"].objectMode());
    input.write(process.env);
    input.end();
  } else if (argv.file) {
    try {
      var filename = (0, _fs.realpathSync)(argv.file);
      (0, _debug["default"])('ezs')("Reading file ".concat(filename, " ..."));
      input = (0, _fs.createReadStream)(filename);
    } catch (e) {
      errlog("Error: ".concat(argv.file, " doesn't exists."));
      process.exit(1);
    }
  } else {
    (0, _debug["default"])('ezs')('Reading standard input...');
    input = process.stdin;
    input.resume();
  }

  var server = Array.isArray(argv.server) ? argv.server : [argv.server];
  var environement = {
    server: server
  };

  var selectFunc = function selectFunc(func, cmds) {
    if (func === 'pipeline') {
      return (0, _["default"])('delegate', {
        commands: cmds
      }, environement);
    }

    return (0, _["default"])('dispatch', {
      commands: cmds,
      server: server,
      environement: environement
    });
  };

  var pipeCommand = function pipeCommand(stream, command, environment) {
    return stream.pipe(_["default"].createCommand(command, environment));
  };

  var runScriptRemote = function runScriptRemote(strm, cmds) {
    (0, _debug["default"])('ezs')('Connecting to server...');
    var runplan = cmds.analyse();
    var usecmds = cmds.getUseCommands();
    var stream0 = usecmds.reduce(pipeCommand, strm);
    return runplan.reduce(function (stream, section) {
      return stream.pipe(selectFunc(section.func, section.cmds));
    }, stream0);
  };

  var runScriptLocal = function runScriptLocal(strm, cmds) {
    return strm.pipe((0, _["default"])('delegate', {
      commands: cmds.get()
    }, environement));
  };

  var runScript = function runScript(serverMode) {
    if (serverMode) {
      return runScriptRemote;
    }

    return runScriptLocal;
  };

  var output = argv._.map(function (arg) {
    var script = (0, _file["default"])(_["default"], arg);

    if (!script) {
      errlog("Error: ".concat(arg, " isn't a file."));
      process.exit(1);
    }

    return script;
  }).map(function (script) {
    return new _commands["default"](_["default"].parseString(script));
  }).reduce(runScript(argv.server), input).pipe(_["default"]["catch"](function (e) {
    return e;
  })).on('error', function (e) {
    errlog(e.message.split('\n').shift());
    process.exit(2);
  }).pipe(_["default"].toBuffer());

  output.on('end', function () {
    process.exit(0);
  });
  output.pipe(process.stdout);
  return argv;
}